function [dat] = read_neuralynx_dma(filename, begsample, endsample, flag);

% READ_NEURALYNX_DMA
%
% Use as
%    [hdr] = read_neuralynx_dma(filename)
%    [dat] = read_neuralynx_dma(filename, begsample, endsample)
%    [dat] = read_neuralynx_dma(filename, begsample, endsample, channel)
%
% The channel specification can be a string with the value
%    'STX', 'PacketID', 'Size', 'TimeStampHi', 'TimeStampLow',
%    'CPU_Status_wd', 'Parallel_in', 'Extra1', 'Extra2', 'Extra3',
%    'Extra4', 'Extra5', 'Extra6', 'Extra7', 'Extra8', 'Extra9',
%    'Extra10'

% Copyright (C) 2005, Robert Oostenveld
%
% $Log: read_neuralynx_dma.m,v $
% Revision 1.8  2006/04/24 12:19:19  roboos
% added support for reading all channels, including the status channels
%
% Revision 1.7  2006/03/14 10:28:45  roboos
% fixed bug in read_neuralynx_dma: when reading a single (special) channel with begsample~=1 it did not skip to the begin sample
%
% Revision 1.6  2006/03/10 12:26:08  roboos
% fixed bug in reading of LastTimeStamp
%
% Revision 1.5  2006/03/06 09:50:45  roboos
% added support for reading a single special channel at a time, using the skip parameter in fread (usefull for Parallel_in)
%
% Revision 1.4  2006/03/02 08:46:11  roboos
% account for the optional 16384 byte header introduced in v4.80
%
% Revision 1.3  2005/10/14 07:05:10  roboos
% added the first and last timestamp to the output header
%
% Revision 1.2  2005/09/09 12:32:00  roboos
% changed spacing and indentation, no functional change
%
% Revision 1.1  2005/09/05 13:05:44  roboos
% new implementation

if nargin<4
  flag = '';
else
  switch flag
    case 'STX'
      channum = 1;
    case 'PacketID'
      channum = 2;
    case 'Size'
      channum = 3;
    case 'TimeStampHi'
      channum = 4;
    case 'TimeStampLow'
      channum = 5;
    case 'CPU_Status_wd'
      channum = 6;
    case 'Parallel_in'
      channum = 7;
    case 'Extra1'
      channum = 8;
    case 'Extra2'
      channum = 9;
    case 'Extra3'
      channum = 10;
    case 'Extra4'
      channum = 11;
    case 'Extra5'
      channum = 12;
    case 'Extra6'
      channum = 13;
    case 'Extra7'
      channum = 14;
    case 'Extra8'
      channum = 15;
    case 'Extra9'
      channum = 16;
    case 'Extra10'
      channum = 17;
    case 'all'
      channum = 1:274;
    otherwise
      error('unknown value in flag');
  end
end

% since Cheetah 4.80 the DMS log files have a 16384 byte header
% determine whether that is the case in this file
fid = fopen(filename, 'rb', 'ieee-le');
buf = fread(fid, 4, 'uchar=>uchar');
if all(buf==35),
  offset = 16384;
else
  offset = 0;
end
fclose(fid);

nboards    =  8;
nchanboard = 32;
blocksize  = nboards*nchanboard+18; % in 4 byte words

if nargin==1
  % these have to be hardcoded, since the DMA logfile does not contain header information
  hdr.Fs           = 32556;     % sampling frequency
  hdr.nChans       = 8*32;      % number of channels
  hdr.nSamples     = inf;       % number of samples per trial
  hdr.nSamplesPre  = 0;         % number of pre-trigger samples in each trial
  hdr.nTrials      = 1;         % number of trials
  hdr.label        = {};        % cell-array with labels of each channel
  for i=1:hdr.nChans
    hdr.label{i} = sprintf('CSC%03d', i);
  end
  fid = fopen(filename, 'rb', 'ieee-le');
  fseek(fid, 0, 'eof');
  % determine the length of the file, expressed in samples
  hdr.nSamples = floor((ftell(fid)-offset)/(blocksize*4));
  % determine the timestamp of the first and of the last sample
  fseek(fid, offset + 0*blocksize, 'bof');
  buf = fread(fid, blocksize, 'int32');
  hdr.FirstTimeStamp = buf(5);
  fseek(fid, offset + (hdr.nSamples-1)*blocksize*4, 'bof');
  buf = fread(fid, blocksize, 'int32');
  hdr.LastTimeStamp = buf(5);
  fclose(fid);
  % only return the header information
  dat = hdr;

elseif nargin==3
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % This is the documentation that I got from Casey Stengel
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % The data is simply a stream of constant size records,  the size of the
  % record is dependant on the number of Input Boards in the system.  8 in your
  % case, there are 32 - 32 bit integers from each Input Board.
  %
  %
  % // Digital Lynx Packets look like this for a 1 board system:
  % // STX (or SOP)    0x00000800
  % // Packet ID       0x00000001
  % // Size            0x0000002A   (# A/D data ints + #extra wds = 32+10)
  % // TimeStamp Hi    1 32 bit word
  % // TimeStamp Low   1 32 bit word
  % // CPU_Status_wd   1 32 bit word
  % // Parallel_in     1 32 bit word
  % // 10 extras      10 32 bit words
  % // A/D data       32 32 bit words per board
  % // CRC             1 32 bit XOR of the entire packet including STX
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % open the file and read the desired bytes
  fid = fopen(filename, 'rb', 'ieee-le');
  % skip the general header
  fseek(fid, offset, 'bof');
  % skip the bytes that do not have to be read
  datoffset = begsample*blocksize*4;
  fseek(fid, datoffset, 'cof');
  % read the data
  buf = fread(fid, [blocksize (endsample-begsample+1)], 'int32=>int32');
  fclose(fid);
  if size(buf,2)<(endsample-begsample+1)
    error('could not read all samples');
  end
  % decipher the data in the buffer, each of these is an int32
  dat.STX           = buf( 1, :);
  dat.PacketID      = buf( 2, :);
  dat.Size          = buf( 3, :);
  dat.TimeStampHi   = buf( 4, :);
  dat.TimeStampLow  = buf( 5, :);
  dat.CPU_Status_wd = buf( 6, :);
  dat.Parallel_in   = buf( 7, :);
  dat.Extras        = buf( 8:17 , :);
  dat.Data          = buf(18:273, :);
  dat.CRC           = buf(274, :);
  % convert the timestamps into numbers that can be used in matlab
  l = dat.TimeStampLow;
  h = dat.TimeStampHi;
  dat.TimeStamp = (double(l) + (l<0)*2^31) + 2^32 * (double(h) + (h<0)*2^31);

elseif ~isempty(channum) && ~strcmp(flag, 'all')
  % read only one channel
  fid = fopen(filename, 'rb', 'ieee-le');
  fseek(fid, offset, 'bof');                % skip the general header
  fseek(fid, (begsample-1)*274*4, 'cof');   % skip to the begin sample
  fseek(fid, (channum-1)*4, 'cof');         % skip to the channel of interest
  % Note that the last block with 274*4 bytes can sometimes be incomplete, which is relevant when endsample=inf
  dat = fread(fid, [1 (endsample-begsample+1)], 'int32=>int32', 273*4);
  fclose(fid);

elseif ~isempty(channum) && strcmp(flag, 'all')
  % read all channels
  fid = fopen(filename, 'rb', 'ieee-le');
  fseek(fid, offset, 'bof');                % skip the general header
  fseek(fid, (begsample-1)*274*4, 'cof');   % skip to the begin sample
  dat = fread(fid, [274 (endsample-begsample+1)], 'int32=>int32');
  fclose(fid);

end
