function [dat] = read_neuralynx_ds(dirname, hdr, begsample, endsample, chanindx);

% READ_NEURALYNX_DS
%
% Use as
%   [hdr] = read_neuralynx_ds(dirname)
%   [dat] = read_neuralynx_ds(dirname, hdr, begsample, endsample, chanindx)
%
% A Neuralynx dataset consists of a directory containing separate files,
% one for each channel. All Neuralynx datafiles starts with a 16k header
% (in ascii format), followed by an arbitrary number of data records. The
% format of the data records depend on the type of data contained in the
% channel (e.g. continuous or spike data).
%
% To read the timestamps of spike waveforms (nse) or clustered spikes (nts),
% the header should contain the fields
%   hdr.FirstTimeStamp
%   hdr.TimeStampPerSample

% Copyright (C) 2006, Robert Oostenveld
%
% $Log: read_neuralynx_ds.m,v $
% Revision 1.1  2006/03/29 14:39:20  roboos
% New implementation, replaces the previous seperate read_neuralynx_header
% and read_neuralynx_data. Instead of reimplementing the reading code, this
% function calls the specialized subfunctions.
%

% get the list of filenames
file = dir(dirname);
file = file(~cell2mat({file.isdir}));
for i=1:length(file)
  filename{i} = fullfile(dirname, file(i).name);
end
clear file

for i=1:length(filename)
  ncs(i) = filetype(filename{i}, 'neuralynx_ncs'); % continuous channels
  nse(i) = filetype(filename{i}, 'neuralynx_nse'); % spike waveform channels
  nts(i) = filetype(filename{i}, 'neuralynx_nts'); % timestamps only
end
filename = filename([find(ncs) find(nse) find(nts)]);

if nargin==1
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  % read only the header of all files in the directory and return
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  clear chanhdr
  for i=1:length(filename)
    dum = getheader(filename{i});
    dum.NumRecords = numrecords(filename{i});
    if dum.NumRecords>0
      dum.FirstTimeStamp = timestamp(filename{i}, 1);
      dum.LastTimeStamp  = timestamp(filename{i}, dum.NumRecords);
    else
      warning(sprintf('the file ''%s'' does not contain any data', filename{i}));
      dum.FirstTimeStamp = nan;
      dum.LastTimeStamp  = nan;
    end
    chanhdr(i) = dum;
  end

  hdr.filename           = filename;
  hdr.label              = {chanhdr(:).NLX_Base_Class_Name};
  hdr.SamplingFrequency  = cell2mat({chanhdr(:).SamplingFrequency});
  hdr.ADBitVolts         = cell2mat({chanhdr(:).ADBitVolts});
  hdr.NumRecords         = cell2mat({chanhdr(:).NumRecords});
  hdr.FirstTimeStamp     = cell2mat({chanhdr(:).FirstTimeStamp});
  hdr.LastTimeStamp      = cell2mat({chanhdr(:).LastTimeStamp});
  % Note that the last timestamp corresponds with the first sample of the last
  % 512-samples record (in case of *.ncs) and not with the last sample in the file.

  if any(hdr.SamplingFrequency~=hdr.SamplingFrequency(1))
    warning('not all channels have the same sampling rate');
  end

  if any(hdr.FirstTimeStamp~=hdr.FirstTimeStamp(1))
    warning('not all channels start at the same time');
  end

  if any(hdr.LastTimeStamp~=hdr.LastTimeStamp(1))
    warning('not all channels end at the same time');
  end

  % remember the detailled header information for each channel
  hdr.chanhdr = chanhdr;

  % the output argument is named "dat"
  dat = hdr;

else
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  % read the data of the selected channels (i.e. files)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  if nargin<5
    % select all channels
    chanindx = 1:length(hdr.label);
  end
  nchan   = length(chanindx);
  nsample = endsample-begsample+1;
  dat     = zeros(nchan, nsample);

  for i=1:nchan
    thischan = chanindx(i);
    thisfile = filename{thischan};
    headersize = 16384;
    switch filetype(thisfile)
    case 'neuralynx_ncs'
      % determine the records that contain the sample numbers of the requested segment
      begrecord  = floor(begsample/512) + 1;
      endrecord  = ceil (endsample/512);
      nrecord    = endrecord-begrecord+1;
      buf        = zeros(1,nrecord*512);
      ncs = read_neuralynx_ncs(thisfile, begrecord, endrecord);
      for j=1:nrecord
        buf(((j-1)*512+1):(j*512)) = ncs.dat{j}(:);
      end
      % copy the selected samples into the output
      begread = (begrecord-1)*512+1;
      endread =  endrecord   *512;
      dat(i,:) = buf((1:nsample) + (begsample-begread));

    case 'neuralynx_nse'
      % read all spike waveforms and timestamps
      nse = read_neuralynx_nse(thisfile);
      % convert the timestamps into samples
      fprintf('%d timstamps\n', length(nse.TimeStamp));
      sample = round((nse.TimeStamp-hdr.FirstTimeStamp)/hdr.TimeStampPerSample+1);
      fprintf('%d rounded-off samples\n', length(sample));
      sample = sample(sample>=begsample & sample<=endsample) - begsample + 1;
      fprintf('%d rounded-off samples in selection\n', length(sample));
      dat(i,sample) = 1;

    case 'neuralynx_nts'
      % read all timestamps
      nts = read_neuralynx_nts(thisfile);
      % convert the timestamps into samples
      sample = round((nts.TimeStamp-hdr.FirstTimeStamp)/hdr.TimeStampPerSample+1);
      sample = sample(sample>=begsample & sample<=endsample) - begsample + 1;
      dat(i,sample) = 1;

    end % switch filetype
  end % for nchan
end % reading data


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for reading the header of a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function hdr = getheader(filename);
fid     = fopen(filename, 'rb', 'ieee-le');
buf     = fread(fid, 16*1024, 'char');
buf     = buf(:)';
nl      = find(buf==10);    % determine the new-lines
cr      = find(buf==13);    % determine the carriage-returns
begchar = [1 nl(1:(end-1))];
endchar = nl - 1;
num     = length(nl);
hdr     = [];
for i=1:num
  line = fliplr(deblank(fliplr(deblank(char(buf(begchar(i):endchar(i)))))));
  if line(1)=='#'
    % line contains a comment
    continue
  else
    % strip the '-' sign
    if line(1)=='-'
      line = line(2:end);
    end
    % replace tabs with spaces
    line(find(line==9)) = ' ';
    % cut into pieces
    item = strread(line, '%s');
    if length(item)==2
      key = item{1};
      val = item{2};
      if any(val(1)=='-01234567989')
        % try to convert to number
        val = str2num(val);
        if isempty(val)
          % revert to the original text
          val = item{2};
        end
      end
      % assign the value to the header structure
      hdr = setfield(hdr, key, val);
    end
  end
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for reading a single timestamp of a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t] = timestamp(filename, num)
fid = fopen(filename, 'rb', 'ieee-le');
headersize = 16384;
switch filetype(filename)
case 'neuralynx_ncs'
  recordsize = 1044;  % in bytes
case 'neuralynx_nse'
  recordsize = 112;   % in bytes
case 'neuralynx_nts'
  recordsize = 8;     % in bytes
end
if ~isinf(num)
  % read the timestamp of the indicated record
  fseek(fid, headersize + (num-1)*recordsize, 'bof');
  t = fread(fid, 1, 'int64');
else
  % read the timestamp of the last record
  fseek(fid, -recordsize, 'eof');
  t = fread(fid, 1, 'int64');
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for determining the number of records in a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t] = numrecords(filename);
fid = fopen(filename, 'rb', 'ieee-le');
headersize = 16384;
switch filetype(filename)
case 'neuralynx_ncs'
  recordsize = 1044;  % in bytes
case 'neuralynx_nse'
  recordsize = 112;   % in bytes
case 'neuralynx_nts'
  recordsize = 8;     % in bytes
end
fseek(fid, 0, 'eof');
t = floor((ftell(fid) - headersize)/recordsize);
fclose(fid);

