function [ncs] = read_neuralynx_ncs(filename, begrecord, endrecord)

% READ_NEURALYNX_ncs reads a single continuous channel file
%
% Use as
%   [ncs] = read_neuralynx_ncs(filename)
%   [ncs] = read_neuralynx_ncs(filename, begrecord, endrecord)

% Copyright (C) 2005, Robert Oostenveld
%
% $Log: read_neuralynx_ncs.m,v $
% Revision 1.8  2006/03/29 15:01:30  roboos
% fix for previous update: only apply the scaling to uV if data has been read
%
% Revision 1.7  2006/03/29 14:43:26  roboos
% scale the output data to uV, using ADBitVolts and an additional 1e6
%
% Revision 1.6  2006/03/23 18:02:13  roboos
% change endrecord from inf into the actual number present
%
% Revision 1.5  2006/03/13 16:02:55  roboos
% added first and last timestamp to the output header
%
% Revision 1.4  2006/03/10 12:32:24  roboos
% preallocate enough memory for all elements (prevent repeated memory-reallocatoin and copying)
%
% Revision 1.3  2005/11/23 08:05:23  roboos
% changed while into for-loop
% added error check for end of file in fseek
% check validity of selected begin and end record
%
% Revision 1.2  2005/09/09 12:29:38  roboos
% changed from dox to unix ascii
% changed the looping over the records
% add the number of records in the output
%
% Revision 1.1  2005/05/20 06:39:35  roboos
% new implementation of single file readers
%

if nargin<2
  begrecord = 1;
end
if nargin<3
  endrecord = inf;
end

% the file starts with a 16*1024 bytes header in ascii, followed by a number of records
hdr = getheader(filename);
fid = fopen(filename, 'rb', 'ieee-le');

% determine the length of the file
fseek(fid, 0, 'eof');
headersize = 16384;
recordsize = 1044;
NRecords   = floor((ftell(fid) - headersize)/recordsize);


if isfinite(endrecord)
  endrecord = NRecords;
end

%temporarily close Neuralynx file
fclose(fid);

% read the timestamp from the first and last record
hdr.FirstTimeStamp = timestamp(filename, 1);
hdr.LastTimeStamp  = timestamp(filename, inf);

%reopen Neuralynx file
fid = fopen(filename, 'rb', 'ieee-le'); 

if begrecord<1
  error('cannot read before the first record');
elseif endrecord>NRecords
  endrecord = NRecords;
end

if endrecord>=begrecord
  % rewind to the first record to be read
  status = fseek(fid, headersize + (begrecord-1)*recordsize, 'bof');
  if status~=0
    error('cannot jump to the requested record');
  end
  
  numrecord = (endrecord-begrecord+1);
  TimeStamp    = zeros(1,numrecord);
  ChanNumber   = zeros(1,numrecord);
  SampFreq     = zeros(1,numrecord);
  NumValidSamp = zeros(1,numrecord);
  Samp         = cell(1,numrecord);

  for k=1:numrecord
    % read a single continuous data record
    TimeStamp(k)    = fread(fid,   1, 'int64');
    ChanNumber(k)   = fread(fid,   1, 'int32');
    SampFreq(k)     = fread(fid,   1, 'int32');
    NumValidSamp(k) = fread(fid,   1, 'int32');
    Samp{k}         = fread(fid, 512, 'int16');
    if length(Samp{k})<512
      error('cannot read beyond end of file');
    else
      % mark the invalid samples
      Samp{k}((NumValidSamp+1):end) = nan;
    end
  end

  % store the record data in the output structure
  ncs.TimeStamp    = TimeStamp;
  ncs.ChanNumber   = ChanNumber;
  ncs.SampFreq     = SampFreq;
  ncs.NumValidSamp = NumValidSamp;
  ncs.dat          = Samp;

  % apply the scaling factor from ADBitVolts and convert to uV
  for k=1:length(ncs.dat)
    ncs.dat{k} = ncs.dat{k} * 1e6 * hdr.ADBitVolts;
  end
end
fclose(fid);

% store the header info in the output structure
ncs.NRecords = NRecords;
ncs.hdr      = hdr;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for reading the header of a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function hdr = getheader(filename);
fid     = fopen(filename, 'rb', 'ieee-le');
buf     = fread(fid, 16*1024, 'char');
buf     = buf(:)';
nl      = find(buf==10);    % determine the new-lines
cr      = find(buf==13);    % determine the carriage-returns
begchar = [1 nl(1:(end-1))];
endchar = nl - 1;
num     = length(nl);
hdr     = [];
for i=1:num
  line = fliplr(deblank(fliplr(deblank(char(buf(begchar(i):endchar(i)))))));
  if line(1)=='#'
    % line contains a comment
    continue
  else
    % strip the '-' sign
    if line(1)=='-'
      line = line(2:end);
    end
    % replace tabs with spaces
    line(find(line==9)) = ' ';
    % cut into pieces
    item = strread(line, '%s');
    if length(item)==2
      key = item{1};
      val = item{2};
      if any(val(1)=='-01234567989')
        % try to convert to number
        val = str2num(val);
        if isempty(val)
          % revert to the original text
          val = item{2};
        end
      end
      % assign the value to the header structure
      hdr = setfield(hdr, key, val);
    end
  end
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for reading a single timestamp of a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t] = timestamp(filename, num)
fid = fopen(filename, 'rb', 'ieee-le');
headersize = 16*1024;
recordsize = 8+3*4+512*2;
if ~isinf(num)
  % read the timestamp of the indicated record
  fseek(fid, headersize + (num-1)*recordsize, 'bof');
  t = fread(fid, 1, 'int64');
else
  % read the timestamp of the last record
  fseek(fid, -recordsize, 'eof');
  t = fread(fid, 1, 'int64');
end
fclose(fid);

