function [dat] = read_neuralynx_sdma(dataset, begsample, endsample, chanindx);

% READ_NEURALYNX_SDMA
%
% Use as
%    [hdr] = read_neuralynx_sdma(dataset)
%    [dat] = read_neuralynx_sdma(dataset, begsample, endsample, chanindx)
%
% The *.sdma dataset is not a formal Neuralynx format, but at the 
% F.C. Donders Centre we use it in conjunction with Neuralynx and
% SPIKEDOWNSAMPLE. The dataset directory contains files, one for each
% channel, each containing a 8-byte header followed by 32-bit integer
% values for all samples.

% Copyright (C) 2006, Robert Oostenveld
%
% $Log: read_neuralynx_sdma.m,v $
% Revision 1.2  2006/12/04 20:26:02  roboos
% change in whitespace
%
% Revision 1.1  2006/04/24 12:21:14  roboos
% new implementation
%

if nargin==1
  % the header has to be hardcoded, since the SDMA dataset does not contain header information
  hdr.Fs           = 32556;     % sampling frequency
  hdr.nChans       = 8*32;      % number of channels
  hdr.nSamples     = inf;       % number of samples per trial
  hdr.nSamplesPre  = 0;         % number of pre-trigger samples in each trial
  hdr.nTrials      = 1;         % number of trials
  hdr.label        = {};        % cell-array with labels of each channel
  for i=1:hdr.nChans
    hdr.label{i} = sprintf('csc%03d', i);
  end

  % determine the number of samples by looking at the file sizes in the directory
  l = dir(dataset);
  s = median(cell2mat({l.bytes}));
  hdr.nSamples = (s-8)/4;

  % determine the first and last timestamp, by reading them from the timestamp channel
  [p, f, x] = fileparts(dataset);
  fid = fopen(fullfile(dataset, [f, '.tsl']), 'rb', 'ieee-le');
  fseek(fid, 8, 'cof');
  hdr.FirstTimeStamp = fread(fid, 1, 'uint32');
  fseek(fid, 8+(hdr.nSamples-1)*4, 'bof');
  hdr.LastTimeStamp = fread(fid, 1, 'uint32');
  fclose(fid);
  
  % return the header
  dat = hdr;
  return

else
  % read all channels, one small chunk at at time, and write it to seperate files
  for i=1:256
    chanext{i} = sprintf('.csc%03d', i);
  end
  [p, f, x] = fileparts(dataset);
  dat = zeros(length(chanindx), endsample-begsample+1);
  for i=1:length(chanindx)
    j = chanindx(i);
    chanfile = fullfile(dataset, [f chanext{j}]);
    fid = fopen(chanfile, 'rb', 'ieee-le');
    % skip the 8-byte channel header
    fseek(fid, 8+begsample-1, 'cof');
    dat(i,:) = fread(fid, endsample-begsample+1, 'int32');
    fclose(fid);
  end
end
